@@ -1,8 +1,8 @@
-### xtable 1.4-2  (2006/10/23)
+### xtable 1.4-3  (2007/01/17)
 ###
 ### Produce LaTeX and HTML tables from R objects.
 ###
-### Copyright 2000-2006 David B. Dahl <dahl@stat.tamu.edu>
+### Copyright 2000-2007 David B. Dahl <dahl@stat.tamu.edu>
 ###
 ### This file is part of the `xtable' library for R and related languages.
 ### It is made available under the terms of the GNU General Public
@@ -47,15 +47,53 @@ label.xtable <- function(x,...) {
 }
 
 "align<-" <- function(x,value) UseMethod("align<-")
+
+# Based on contribution from Jonathan Swinton <jonathan@swintons.net> in e-mail dated Wednesday, January 17, 2007
+.alignStringToVector <- function(aString) {
+  # poor mans parsing - separating string of form "l{2in}llr|p{1in}c|{1in}"
+  # into "l{2in}" "l"      "l"      "r"      "|"      "p{1in}" "c"      "|{1in}"
+  aString.Align <- character(0);
+  aString.Width <- character(0);
+  wString <- aString
+  while( nchar(wString)>0) {
+    aString.Align <- c(aString.Align,substr(wString,1,1))
+    # is it followed by a brace?
+    thisWidth <- ""
+    if ( nchar(wString)>1 & substr(wString,2,2)=="{") {
+      beforeNextBrace <- regexpr("[^\\]\\}",wString)
+      if (beforeNextBrace <0 ) {
+        stop("No closing } in align string")
+      }
+      thisWidth <- substr(wString,2,beforeNextBrace+1)
+      wString <- substr(wString,beforeNextBrace+2,nchar(wString))
+    } else {
+      wString <- substr(wString,2,nchar(wString))
+    }
+    aString.Width <- c(aString.Width,thisWidth)
+  }
+
+  alignAllowed <- c("l","r","p","c","|") 
+  
+  if (any( !(aString.Align %in% alignAllowed))) {
+    warning("Nonstandard alignments in align string")
+  }
+  res <- paste(aString.Align,aString.Width,sep="")
+  res
+}
+#.alignStringToVector ("l{2in}llr|p{1in}c|{1in}")
+#.alignStringToVector ("l{2in}llr|p{1in}c|")
+#.alignStringToVector ("{2in}llr|p{1in}c|") # latex syntax error, but gives wrong alignment
+#.alignStringToVector("llllp{3cm}")
+
 "align<-.xtable" <- function(x,value) {
 # Based on contribution from Benno <puetz@mpipsykl.mpg.de> in e-mail dated Wednesday, December 01, 2004
+# Based on contribution from Jonathan Swinton <jonathan@swintons.net> in e-mail dated Wednesday, January 17, 2007
   # cat("%",value,"\n")
   if ( (!is.null(value)) && ( is.character(value) ) && ( length(value) == 1 ) && ( nchar(value) > 1 ) ) {
-    value <- strsplit(value,"")[[1]]
-  }
-  if (!all(!is.na(match(value,c("r","l","c","|"))))){
-    stop("\"align\" must be containing elements of {\"r\",\"l\",\"c\",\"|\"}")
-  }
+        value <- .alignStringToVector(value)
+  } # That should have checked we had only lrcp|
+    # but what if the "if statement" is false?
+    # For simplicity, deleting check present in version 1.4-2 and earlier.
   c.value <- if (any(!is.na(match(value,"|")))) {
                 value[-which(value=='|')]
              } else {
@@ -82,9 +120,10 @@ align.xtable <- function(x,...) {
         " ( nrow( x )" )
     }
   } else {
-    if( length( value ) != ncol(x)+1 ) {
-      stop( "if argument 'digits' is a vector, it must have length equal",
-        " to", ncol(x)+1, " ( ncol(x) + 1 )" )
+    if( length(value)==1 ) { value <- rep(value, ncol(x)+1) } 
+    if( length( value ) >1 & length( value ) != ncol(x)+1 ) {
+      stop( "if argument 'digits' is a vector of length more than one, it must have length equal",
+        " to ", ncol(x)+1, " ( ncol(x) + 1 )" )
     }
   }
   if (!is.numeric(value))
